/*
 * NFARun.cpp
 *
 *  Created on: 14. 2. 2014
 *      Author: tomas
 */

#include "NFARun.h"
#include <iostream>

using namespace alib;
using namespace automaton;
using namespace std;

NFARun::NFARun( const FSM & fsm ) : m_fsm( fsm )
{

}

bool NFARun::simulate( const list<Symbol> & string ) const
{
    if( ! checkAlphabet( string ) )
        throw AlibException( "String alphabet does not match automaton alphabet." );

    for( const auto & state : m_fsm.getInitialStates( ) )
        if( configuration( state, string ) )
            return true;

    return false;
}

bool NFARun::checkAlphabet( const list<Symbol> & string ) const
{
    const auto & alphabet = m_fsm.getInputAlphabet( );
    for( const auto & symbol : string )
        if( alphabet.find( symbol ) == alphabet.end( ) )
            return false;

    return true;
}

bool NFARun::configuration( const State & state, const list<Symbol> & string ) const
{
    if( string.empty( ) )
        return m_fsm.getFinalStates( ).find( state ) != m_fsm.getFinalStates( ).end( );

    const Symbol & inputSymbol = string.front( );

    list<Symbol> newString = string;
    newString.pop_front( );

    for( const auto & transition : m_fsm.getTransitionsFromState( state ) )
        if( transition.getInput( ) == inputSymbol && configuration( transition.getTo( ), newString ) )
            return true;

    return false;
}

State NFARun::createUniqueState( const string & desiredName ) const
{
    //TODO: No need for this method after utils from aconversions project is merged into alib

    State s( desiredName );
    if( m_fsm.getStates( ).find( s ) == m_fsm.getStates( ).end( ) )
        return s;

    int i = 0;
    while( true )
    {
        State s( desiredName + to_string( i ++ ) );

        // state not present in totalFSM
        if( m_fsm.getStates( ).find( s ) == m_fsm.getStates( ).end( ) )
            return s;
    }
    throw AlibException( "WordAcceptance - Unable to create trash state" );
}
