/*
 * DotConverter.h
 *
 *  Created on: Apr 1, 2013
 *      Author: Martin Zak
 */

#ifndef DOT_CONVERTER_H_
#define DOT_CONVERTER_H_

#include <ostream>

#include <automaton/Automaton.h>

#include <map>
#include <utility>

class DotConverter : public automaton::VisitableAutomatonBase::const_visitor_type {
	void Visit(void*, const automaton::UnknownAutomaton& automaton) const;
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	static void transitions(const automaton::UnknownAutomaton& fsm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::EpsilonNFA& fsm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::NFA& fsm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::DFA& fsm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::ExtendedNFA& fsm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::CompactNFA& fsm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::DPDA& pda, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::SinglePopDPDA& tm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::InputDrivenNPDA& pda, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::VisiblyPushdownDPDA& tm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::VisiblyPushdownNPDA& tm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::RealTimeHeightDeterministicDPDA& tm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::RealTimeHeightDeterministicNPDA& tm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::NPDA& pda, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::SinglePopNPDA& tm, const std::map<automaton::State, int>& states, std::ostream& out);
	static void transitions(const automaton::OneTapeDTM& tm, const std::map<automaton::State, int>& states, std::ostream& out);

	static const DotConverter instance;
public:
	static void convert(const automaton::Automaton& a, std::ostream& out);

	static void convert(const automaton::UnknownAutomaton& a, std::ostream& out);
	static void convert(const automaton::EpsilonNFA& a, std::ostream& out);
	static void convert(const automaton::NFA& a, std::ostream& out);
	static void convert(const automaton::DFA& a, std::ostream& out);
	static void convert(const automaton::ExtendedNFA& a, std::ostream& out);
	static void convert(const automaton::CompactNFA& a, std::ostream& out);
	static void convert(const automaton::DPDA& a, std::ostream& out);
	static void convert(const automaton::SinglePopDPDA& a, std::ostream& out);
	static void convert(const automaton::InputDrivenNPDA& a, std::ostream& out);
	static void convert(const automaton::VisiblyPushdownDPDA& a, std::ostream& out);
	static void convert(const automaton::VisiblyPushdownNPDA& a, std::ostream& out);
	static void convert(const automaton::RealTimeHeightDeterministicDPDA& a, std::ostream& out);
	static void convert(const automaton::RealTimeHeightDeterministicNPDA& a, std::ostream& out);
	static void convert(const automaton::NPDA& a, std::ostream& out);
	static void convert(const automaton::SinglePopNPDA& a, std::ostream& out);
	static void convert(const automaton::OneTapeDTM& a, std::ostream& out);
};

#endif /* DOT_CONVERTER_H_ */
