/*
 * AutomatonDiff.cpp
 *
 *  Created on: Apr 1, 2013
 *      Author: martin
 */

#include "AutomatonDiff.h"
#include "automaton/common/State.h"

#include "automaton/Automaton.h"

#include <set>
#include <map>
#include <list>
#include <utility>
#include <vector>
#include <typeinfo>
#include <iostream>
#include <algorithm>

#include "automaton/UnknownAutomaton.h"
#include "automaton/FSM/DFA.h"
#include "automaton/FSM/NFA.h"
#include "automaton/FSM/EpsilonNFA.h"
#include "automaton/FSM/ExtendedNFA.h"
#include "automaton/FSM/CompactNFA.h"
#include "automaton/PDA/NPDA.h"
#include "automaton/PDA/SinglePopNPDA.h"
#include "automaton/TM/OneTapeDTM.h"

#include "exception/AlibException.h"

bool AutomatonDiff::testDiff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b) {
	return  	a.getBlankSymbol()    == b.getBlankSymbol()    &&
			a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputSymbols()   == b.getInputSymbols()   &&
//			a.getStackSymbols()   == b.getStackSymbols()   &&
			a.getInitialSymbols() == b.getInitialSymbols() &&
			a.getStates()         == b.getStates()         &&
//			a.getTapeSymbols()    == b.getTapeSymbols()    &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::DFA& a, const automaton::DFA& b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::NFA& a, const automaton::NFA& b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::CompactNFA& a, const automaton::CompactNFA& b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::NPDA& a, const automaton::NPDA& b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
//			a.getStackAlphabet()  == b.getStackAlphabet()  &&
			a.getInitialSymbols() == b.getInitialSymbols() &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b) {
	return  	a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialStates()  == b.getInitialStates()  &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
//			a.getStackAlphabet()  == b.getStackAlphabet()  &&
			a.getInitialSymbols() == b.getInitialSymbols() &&
			a.getStates()         == b.getStates()         &&
			a.getTransitions()    == b.getTransitions()    ;
}

bool AutomatonDiff::testDiff(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b) {
	return  	a.getBlankSymbol()    == b.getBlankSymbol()    &&
			a.getFinalStates()    == b.getFinalStates()    &&
			a.getInitialState()   == b.getInitialState()   &&
//			a.getInputAlphabet()  == b.getInputAlphabet()  &&
			a.getStates()         == b.getStates()         &&
//			a.getTapeAlphabet()   == b.getTapeAlphabet()   &&
			a.getTransitions()    == b.getTransitions()    ;
}

template <class T> void AutomatonDiff::setDiff(const std::set<T> a, const std::set<T> b) {
	std::set<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::set<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::set<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::set<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T> void AutomatonDiff::listDiff(const std::list<T> a, const std::list<T> b) {
	std::list<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::list<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::list<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::list<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T, class R> void AutomatonDiff::mapDiff(const std::map<T, R> a, const std::map<T, R> b) {
	std::map<T, R> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::map<T, R> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::map<T, R>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << iter->first << ", " << iter->second << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::map<T, R>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "< " << iter->first << ", " << iter->second << std::endl;
	}
}

void AutomatonDiff::printDiff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getBlankSymbol() != b.getBlankSymbol()) {
		std::cout << "Blank symbol" << std::endl;

		std::cout << "< " << a.getBlankSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getBlankSymbol() << std::endl;
	}

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()){
		std::cout << "InitialStates" << std::endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputSymbols() != b.getInputSymbols()) {
		std::cout << "InputSymbols" << std::endl;

		AutomatonDiff::setDiff(a.getInputSymbols(), b.getInputSymbols());
	}

	if(a.getStackSymbols() != b.getStackSymbols()) {
		std::cout << "StackSymbols" << std::endl;

		AutomatonDiff::setDiff(a.getStackSymbols(), b.getStackSymbols());
	}

	if(a.getInitialSymbols() != b.getInitialSymbols()) {
		std::cout << "InitialSymbols" << std::endl;

		AutomatonDiff::setDiff(a.getInitialSymbols(), b.getInitialSymbols());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTapeSymbols() != b.getTapeSymbols()) {
		std::cout << "TapeSymbols" << std::endl;

		AutomatonDiff::setDiff(a.getTapeSymbols(), b.getTapeSymbols());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::setDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::DFA& a, const automaton::DFA& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::NFA& a, const automaton::NFA& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()) {
		std::cout << "InitialStates" << std::endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()) {
		std::cout << "InitialStates" << std::endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()) {
		std::cout << "InitialStates" << std::endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::CompactNFA& a, const automaton::CompactNFA& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()) {
		std::cout << "InitialStates" << std::endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::NPDA& a, const automaton::NPDA& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()){
		std::cout << "InitialStates" << std::endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStackAlphabet() != b.getStackAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getStackAlphabet(), b.getStackAlphabet());
	}

	if(a.getInitialSymbols() != b.getInitialSymbols()) {
		std::cout << "InitialSymbols" << std::endl;

		AutomatonDiff::setDiff(a.getInitialSymbols(), b.getInitialSymbols());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialStates() != b.getInitialStates()){
		std::cout << "InitialStates" << std::endl;

		AutomatonDiff::setDiff(a.getInitialStates(), b.getInitialStates());
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStackAlphabet() != b.getStackAlphabet()) {
		std::cout << "StackAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getStackAlphabet(), b.getStackAlphabet());
	}

	if(a.getInitialSymbols() != b.getInitialSymbols()) {
		std::cout << "InitialSymbols" << std::endl;

		AutomatonDiff::setDiff(a.getInitialSymbols(), b.getInitialSymbols());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

void AutomatonDiff::printDiff(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b) {
	std::cout << "AutomataDiffer" << std::endl;

	if(a.getBlankSymbol() != b.getBlankSymbol()) {
		std::cout << "Blank symbol" << std::endl;

		std::cout << "< " << a.getBlankSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getBlankSymbol() << std::endl;
	}

	if(a.getFinalStates() != b.getFinalStates()){
		std::cout << "FinalStates" << std::endl;

		AutomatonDiff::setDiff(a.getFinalStates(), b.getFinalStates());
	}

	if(a.getInitialState() != b.getInitialState()) {
		std::cout << "Initial state" << std::endl;

		std::cout << "< " << a.getInitialState() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialState() << std::endl;
	}

	if(a.getInputAlphabet() != b.getInputAlphabet()) {
		std::cout << "InputAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getInputAlphabet(), b.getInputAlphabet());
	}

	if(a.getStates() != b.getStates()) {
		std::cout << "States" << std::endl;

		AutomatonDiff::setDiff(a.getStates(), b.getStates());
	}

	if(a.getTapeAlphabet() != b.getTapeAlphabet()) {
		std::cout << "TapeAlphabet" << std::endl;

		AutomatonDiff::setDiff(a.getTapeAlphabet(), b.getTapeAlphabet());
	}

	if(a.getTransitions() != b.getTransitions()) {
		std::cout << "Transitions" << std::endl;

		AutomatonDiff::mapDiff(a.getTransitions(), b.getTransitions());
	}
}

int AutomatonDiff::diff(const automaton::UnknownAutomaton& a, const automaton::UnknownAutomaton& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::DFA& a, const automaton::DFA& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::NFA& a, const automaton::NFA& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::EpsilonNFA& a, const automaton::EpsilonNFA& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::ExtendedNFA& a, const automaton::ExtendedNFA& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::CompactNFA& a, const automaton::CompactNFA& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::NPDA& a, const automaton::NPDA& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::SinglePopNPDA& a, const automaton::SinglePopNPDA& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

int AutomatonDiff::diff(const automaton::OneTapeDTM& a, const automaton::OneTapeDTM& b) {
	if(!AutomatonDiff::testDiff(a, b)) {
	  AutomatonDiff::printDiff(a, b);
	  return 1;
	} else {
	  return 0;
	}
}

template<typename T>
int AutomatonDiff::dynamicDiff(const automaton::Automaton& a, const automaton::Automaton& b) {
	const T* ap = dynamic_cast<const T*>(&a.getData());
	const T* bp = dynamic_cast<const T*>(&b.getData());

	if(ap && bp) {
		return diff(*ap, *bp);
	} else if(ap || bp) {
		std::cout << "AutomataDifferByType" << std::endl;
		return 1;
	} else {
		return -1;
	}

}

int AutomatonDiff::diff(const automaton::Automaton& a, const automaton::Automaton& b) {
	int res;

	res = dynamicDiff<automaton::UnknownAutomaton>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::DFA>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::NFA>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::EpsilonNFA>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::ExtendedNFA>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::CompactNFA>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::NPDA>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::SinglePopNPDA>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	res = dynamicDiff<automaton::OneTapeDTM>(a, b);
	if(res == 0) return 0;
	if(res == 1) return 1;

	throw exception::AlibException("Unknown automata type");
}
