/*
 * BrzozowskiDerivation.h
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#ifndef RE2FA_BRZOZOWSKIDERIVATION_H_
#define RE2FA_BRZOZOWSKIDERIVATION_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

#include <automaton/Automaton.h>
#include <automaton/FSM/NFA.h>

namespace conversions
{

namespace re2fa
{

/**
 * Converts regular expression to finite automaton using BrzozowskiDerivation algorithm (derivations of regular expressions).
 * Source: Melichar 2.110
 */
class BrzozowskiDerivation : public regexp::VisitableRegExpBase::const_visitor_type
{
public:
	/**
	 * Performs conversion.
	 * @return FSM equivalent to original regular expression.
	 */
	static automaton::Automaton convert(const regexp::RegExp& regexp);

	template<class T>
	static automaton::NFA convert(const T& regexp);

private:
	void Visit(void* , const regexp::FormalRegExp& regexp) const;
	void Visit(void* , const regexp::UnboundedRegExp& regexp) const;

	static const BrzozowskiDerivation BRZOZOWSKI_DERIVATION;
};

} /* namespace re2fa */

} /* namespace conversions */

#endif /* RE2FA_BRZOZOWSKIDERIVATION_H_ */
