/*
 * RightToLeftRegularGrammar.h
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#ifndef RIGHT_TO_LEFT_REGULAR_GRAMMAR_H_
#define RIGHT_TO_LEFT_REGULAR_GRAMMAR_H_

#include <grammar/Grammar.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

namespace conversions
{

namespace rg2rg
{

/**
 * Converts right regular grammar to left regular grammar.
 */
class RightToLeftRegularGrammar : public grammar::VisitableGrammarBase::const_visitor_type
{
public:
    /**
     * Performs conversion.
     * @param grammar Right regular grammar to convert
     * @return left regular grammar which is equivalent to source right regular grammar.
     */
    static grammar::LeftRG convert(const grammar::Grammar& grammar);
    static grammar::LeftRG convert(const grammar::RightRG& grammar);

private:
    void Visit(void*, const grammar::UnknownGrammar& grammar) const;

    void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
    void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

    void Visit(void*, const grammar::CSG& grammar) const;
    void Visit(void*, const grammar::NonContractingGrammar& grammar) const;

    void Visit(void*, const grammar::CNF& grammar) const;
    void Visit(void*, const grammar::CFG& grammar) const;
    void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
    void Visit(void*, const grammar::GNF& grammar) const;
    void Visit(void*, const grammar::LG& grammar) const;

    void Visit(void*, const grammar::RightRG& grammar) const;
    void Visit(void*, const grammar::LeftRG& grammar) const;
    void Visit(void*, const grammar::RightLG& grammar) const;
    void Visit(void*, const grammar::LeftLG& grammar) const;

    static const RightToLeftRegularGrammar RIGHT_TO_LEFT_REGULAR_GRAMMAR;
};

} /* namespace rg2rg */

} /* namespace conversions */

#endif /* RIGHT_TO_LEFT_REGULAR_GRAMMAR_H_ */
