/*
 * FSMEpsilonRemover.h
 *
 *  Created on: 16. 1. 2014
 *      Author: Tomas Pecka
 */

#ifndef FMS_EPSILON_REMOVER_H_
#define FMS_EPSILON_REMOVER_H_

#include <map>
#include <algorithm>

#include <automaton/Automaton.h>

#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>
#include <exception/AlibException.h>

namespace epsilon {

class FSMEpsilonRemover : public automaton::VisitableAutomatonBase::const_visitor_type {
public:
	static automaton::Automaton remove( const automaton::Automaton & automaton );

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::NFA remove( const automaton::EpsilonNFA & fsm );
	static automaton::NFA remove( const automaton::NFA & fsm );
	static automaton::DFA remove( const automaton::DFA & fsm );

private:
	void Visit(void*, const automaton::UnknownAutomaton& automaton) const;
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;

	static const FSMEpsilonRemover FSM_EPSILON_REMOVER;
};

} /* namespace epsilon */

#endif /* FMS_EPSILON_REMOVER_H_ */
