/*
 * RightRegularEquationSolver.cpp
 *
 *  Created on: 4. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "RightRegularEquationSolver.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"

#include "../regexp/RegExpOptimize.h"

namespace equations {

regexp::UnboundedRegExp RightRegularEquationSolver::eliminate(void) {
	regexp::RegExpOptimize opt;

	for(auto itA = nonterminalSymbolsByDepth.rbegin(); itA != nonterminalSymbolsByDepth.rend(); itA ++) {
		const alphabet::Symbol& a = * itA;

		/*
		 * Apply Arden's Lemma
		 * A = 0A + 1B + 2C
		 * => A = 0*1B + 0*2C
		 */
		regexp::UnboundedRegExpIteration loop(std::move(equationTransition.find(std::make_pair(a, a))->second));
		opt.optimize(loop);

		// for all transitions from A apply Arden's Lemma
		for(auto itB = std::next(itA) ; itB != nonterminalSymbolsByDepth.rend(); itB ++) {
			const alphabet::Symbol& b = * itB;
			regexp::UnboundedRegExpConcatenation concat;
			concat.appendElement(loop);
			concat.appendElement(std::move(equationTransition.find(std::make_pair(a,b))->second));
			regexp::UnboundedRegExpAlternation alt;
			alt.appendElement(std::move(concat));
			equationTransition.find(std::make_pair(a, b))->second = std::move(alt);
			opt.optimize(equationTransition.find(std::make_pair(a, b))->second);
		}
		regexp::UnboundedRegExpConcatenation concat;
		concat.appendElement(std::move(loop));
		concat.appendElement(std::move(equationFinal.find(a)->second));
		regexp::UnboundedRegExpAlternation alt;
		alt.appendElement(std::move(concat));
		equationFinal.find(a)->second = std::move(alt);
		opt.optimize(equationFinal.find(a)->second);

		/*
		 * eliminate A from rest of the equations using this pattern:
		 * B->C = B->C + concatenate(B->A, A->C)
		 */
		for(auto itB = std::next(itA); itB != nonterminalSymbolsByDepth.rend(); itB ++) {
			const alphabet::Symbol& b = * itB;

			for(auto itC = std::next(itA); itC != nonterminalSymbolsByDepth.rend(); itC ++) {
				const alphabet::Symbol& c = * itC;

				regexp::UnboundedRegExpConcatenation concat;
				concat.appendElement(equationTransition.find(std::make_pair(b, a))->second);
				concat.appendElement(equationTransition.find(std::make_pair(a, c))->second);
				regexp::UnboundedRegExpAlternation alt;
				alt.appendElement(std::move(equationTransition.find(std::make_pair(b, c))->second));
				alt.appendElement(std::move(concat));
				equationTransition.find(std::make_pair(b, c))->second = std::move(alt);
				opt.optimize(equationTransition.find(std::make_pair(b, c))->second);
			}

			regexp::UnboundedRegExpConcatenation concat;
			concat.appendElement(std::move(equationTransition.find(std::make_pair(b, a))->second));
			concat.appendElement(equationFinal.find(a)->second);
			regexp::UnboundedRegExpAlternation alt;
			alt.appendElement(std::move(equationFinal.find(b)->second));
			alt.appendElement(std::move(concat));
			equationFinal.find(b)->second = std::move(alt);
			opt.optimize(equationFinal.find(b)->second);
		}
	}

	return opt.optimize(regexp::UnboundedRegExp(std::move( equationFinal.find(*nonterminalSymbolsByDepth.begin())->second)));
}

} /* namespace equations */
