/*
 * GrammarPropertiesCFG.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef GRAMMAR_PROPERTIES_CFG_H_
#define GRAMMAR_PROPERTIES_CFG_H_

#include <algorithm>
#include <deque>
#include <set>

#include <exception/AlibException.h>

#include <alphabet/Symbol.h>

namespace grammar {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class GrammarPropertiesCFG {
public:
	/**
	 * Implements steps 1 through 3 in Melichar 3.6
	 */
	template<class T>
	static std::set<alphabet::Symbol> getProductiveNonterminals( const T & grammar );

	/**
	 * Implements
	 */
	template<class T>
	static std::set<alphabet::Symbol> getUnreachableSymbols( const T & grammar );

	/**
	 * Retrieve all nullable nonterminals from grammar
	 * Nullable nonterminal is such nonterminal A for which holds that A ->^* \eps
	 *
	 * Source: Melichar, algorithm 2.4, step 1
	 *
	 * @param grammar grammar
	 * @return set of nullable nonterminals from grammar
	 */
	template<class T>
	static std::set<alphabet::Symbol> getNullableNonterminals(const T& grammar);

	/**
	 * Retrieves set N = {B : A->^* B} for given grammar and nonterminal
	 *
	 * Source: Melichar, algorithm 2.6, step 1
	 *
	 * @param grammar grammar
	 * @param nonterminal nonterminal
	 * @return set of nonterminals for which we can be derived from giveUnitRuleCyclen nonterminals in finite number of steps
	 */
	template<class T>
	static std::set<alphabet::Symbol> getNonterminalUnitRuleCycle(const T& grammar, const alphabet::Symbol& nonterminal);
};

} /* namespace grammar */

#endif /* GRAMMAR_PROPERTIES_CFG_H_ */
