#include "FSMTotalTest.h"

#include <automaton/FSM/DFA.h>

#include "automaton/FSMTotal.h"
#include "normalize/dfa/NormalizeDFA.h"
#include "trim/automaton/TrimFSM.h"

CPPUNIT_TEST_SUITE_REGISTRATION( TotalTest );

void TotalTest::setUp() {
}

void TotalTest::tearDown() {
}

void TotalTest::testTotal() {
	automaton::State q0("q0");
	automaton::State q1("q1");
	automaton::State q2("q2");

	alphabet::Symbol a = alphabet::symbolFrom('a');
	alphabet::Symbol b = alphabet::symbolFrom('b');
	alphabet::Symbol c = alphabet::symbolFrom('c');

	automaton::DFA automaton(q0);
	automaton.setStates({q0, q1, q2});
	automaton.setFinalStates({q0, q1, q2});
	automaton.setInputSymbols({a, b, c});

	automaton.addTransition(q0, a, q0);
	automaton.addTransition(q0, b, q1);
	automaton.addTransition(q0, c, q2);
	automaton.addTransition(q1, b, q1);
	automaton.addTransition(q1, c, q2);
	automaton.addTransition(q2, c, q2);

	automaton::DFA totalAutomaton = automaton::FSMTotal::total(automaton);
	CPPUNIT_ASSERT(totalAutomaton.isTotal());

	automaton::DFA trimmedAutomaton = trim::TrimFSM::trim(automaton);
	automaton::DFA trimmedTotalAutomaton = trim::TrimFSM::trim(totalAutomaton);

	CPPUNIT_ASSERT(normalize::NormalizeDFA::normalize(trimmedAutomaton) == normalize::NormalizeDFA::normalize(trimmedTotalAutomaton));
}
