/*
 * AutomatonFromXMLParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: Martin Zak
 */

#ifndef AUTOMATON_FROM_XML_PARSER_H_
#define AUTOMATON_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include "Automaton.h"
#include "AutomatonFeatures.h"
#include "UnknownAutomaton.h"
#include "FSM/EpsilonNFA.h"
#include "FSM/NFA.h"
#include "FSM/DFA.h"
#include "FSM/CompactNFA.h"
#include "FSM/ExtendedNFA.h"
#include "PDA/DPDA.h"
#include "PDA/SinglePopDPDA.h"
#include "PDA/InputDrivenNPDA.h"
#include "PDA/VisiblyPushdownDPDA.h"
#include "PDA/VisiblyPushdownNPDA.h"
#include "PDA/RealTimeHeightDeterministicDPDA.h"
#include "PDA/RealTimeHeightDeterministicNPDA.h"
#include "PDA/NPDA.h"
#include "PDA/SinglePopNPDA.h"
#include "TM/OneTapeDTM.h"

#include <list>
#include <set>
#include "../std/variant.hpp"
#include "../sax/Token.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace automaton {

/**
 * Parser used to get general FSM or EpsilonNFA, NFA, DFA from XML parsed into list of Tokens.
 */
class AutomatonFromXMLParser : public sax::FromXMLParserHelper {
	std::set<State> parseStates(std::list<sax::Token> &input) const;
	State parseState(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseInputAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseCallInputAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseReturnInputAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseLocalInputAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseStackAlphabet(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseInitialStackSymbols(std::list<sax::Token> &input) const;
	alphabet::Symbol parseInitialStackSymbol(std::list<sax::Token> &input) const;
	std::set<alphabet::Symbol> parseTapeAlphabet(std::list<sax::Token> &input) const;
	State parseInitialState(std::list<sax::Token> &input) const;
	std::set<State> parseInitialStates(std::list<sax::Token> &input) const;
	std::set<State> parseFinalStates(std::list<sax::Token> &input) const;
	alphabet::Symbol parseBlankSymbol(std::list<sax::Token> &input) const;
	alphabet::Symbol parseBottomOfTheStackSymbol(std::list<sax::Token> &input) const;
	void parseInputToPushdownStoreOperation(std::list<sax::Token>& input, automaton::InputDrivenNPDA& automaton) const;

	State parseTransitionFrom(std::list<sax::Token>& input) const;
	State parseTransitionTo(std::list<sax::Token>& input) const;
	Shift parseTransitionShift(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseTransitionPop(std::list<sax::Token>& input) const;
	alphabet::Symbol parseTransitionSinglePop(std::list<sax::Token>& input) const;
	std::vector<alphabet::Symbol> parseTransitionPush(std::list<sax::Token>& input) const;
	alphabet::Symbol parseTransitionSinglePush(std::list<sax::Token>& input) const;

	template<class T>
	void parseTransitions(std::list<sax::Token> &input, T& automaton) const;

	void parseTransition(std::list<sax::Token>& input, UnknownAutomaton& automaton) const;
	void parseTransition(std::list<sax::Token>& input, EpsilonNFA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, NFA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, DFA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, CompactNFA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, ExtendedNFA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, DPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, SinglePopDPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, InputDrivenNPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, VisiblyPushdownDPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, VisiblyPushdownNPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, RealTimeHeightDeterministicDPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, RealTimeHeightDeterministicNPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, NPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, SinglePopNPDA& automaton) const;
	void parseTransition(std::list<sax::Token>& input, OneTapeDTM& automaton) const;

	alphabet::Symbol parseTransitionInputSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionInputEpsilonSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionInputBlankEpsilonSymbol(std::list<sax::Token> &input) const;
	string::LinearString parseTransitionInputString(std::list<sax::Token> &input) const;
	regexp::RegExp parseTransitionInputRegexp(std::list<sax::Token> &input) const;

	alphabet::Symbol parseTransitionOutputSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionOutputEpsilonSymbol(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseTransitionOutputBlankEpsilonSymbol(std::list<sax::Token> &input) const;

	alphabet::Symbol parseSymbolIO(std::list<sax::Token> &input) const;
	std::variant<string::Epsilon, alphabet::Symbol> parseEpsilonSymbolIO(std::list<sax::Token> &input) const;

	Automaton parseAutomaton(std::list<sax::Token>& input) const;
	Automaton parseAutomaton(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;

	UnknownAutomaton parseUnknownAutomaton(std::list<sax::Token>& input) const;
	EpsilonNFA parseEpsilonNFA(std::list<sax::Token>& input) const;
	NFA parseNFA(std::list<sax::Token>& input) const;
	DFA parseDFA(std::list<sax::Token>& input) const;
	CompactNFA parseCompactNFA(std::list<sax::Token>& input) const;
	ExtendedNFA parseExtendedNFA(std::list<sax::Token>& input) const;
	DPDA parseDPDA(std::list<sax::Token>& input) const;
	SinglePopDPDA parseSinglePopDPDA(std::list<sax::Token>& input) const;
	InputDrivenNPDA parseInputDrivenNPDA(std::list<sax::Token>& input) const;
	VisiblyPushdownDPDA parseVisiblyPushdownDPDA(std::list<sax::Token>& input) const;
	VisiblyPushdownNPDA parseVisiblyPushdownNPDA(std::list<sax::Token>& input) const;
	RealTimeHeightDeterministicDPDA parseRealTimeHeightDeterministicDPDA(std::list<sax::Token>& input) const;
	RealTimeHeightDeterministicNPDA parseRealTimeHeightDeterministicNPDA(std::list<sax::Token>& input) const;
	NPDA parseNPDA(std::list<sax::Token>& input) const;
	SinglePopNPDA parseSinglePopNPDA(std::list<sax::Token>& input) const;
	OneTapeDTM parseOneTapeDTM(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::api;
public:
	bool first(std::list<sax::Token>& input) const;
};

} /* namespace automaton */

#endif /* AUTOMATON_FROM_XML_PARSER_H_ */

