/*
 * .h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef MULTI_INITIAL_STATE_H_
#define MULTI_INITIAL_STATE_H_

#include "States.h"
#include <set>
#include "State.h"

namespace automaton {

/**
 * Base class for nondeterministic automata.
 */
class MultiInitialStates : public States {
protected:
	std::set<State> initialStates;
public:
	/**
	 * Adds the State to the initial states.
	 * @param state State to add
	 * @throws AutomatonException when state is not present
	 * in the automaton or when it's already initial state
	 */
	bool addInitialState(const State& state);

	/**
	 * Sets the initial state of the automaton.
	 * @param state State to add
	 * @throws AutomatonException when state is not present
	 * in the automaton or when it's already initial state
	 */
	void setInitialStates(const std::set<State>& states);

	/**
	 * Removes the state from the initial states.
	 * @param state State to remove
	 * @throws AutomatonException when State is not initial state
	 */
	bool removeInitialState(const State& state);

	/**
	 * @return initial states
	 */
	const std::set<State>& getInitialStates() const;
};

} /* namespace automaton */

#endif /* MULTI_INITIAL_STATE_H_ */

