/*
 * Base.h
 *
 *  Created on: Apr 05, 2014
 *      Author: Jan Travnicek
 */

#ifndef BASE_H_
#define BASE_H_

#include <typeinfo>
#include <ostream>

namespace alib {

template<int ID>
class base_base {
public:
	template<typename T>
	int typeId(const T&) const {
		return ID;
	}

	virtual int selfTypeId() const = 0;
};

template<int ID, typename... Types>
class base_helper;

template<int ID, typename T>
class base_helper<ID, T> : public base_base<ID + 1> {
public:
	virtual int typeId(const T &) const {
		return ID;
	}

	virtual bool operator==(const T &) const {
		return false;
	}

	virtual bool operator<(const T & other) const {
		return this->selfTypeId() < typeId(other);
	}

	virtual ~base_helper() noexcept {

	}
};

template<int ID, typename T, typename... Types>
class base_helper<ID, T, Types...> : public base_helper<ID + 1, Types...> {
public:
	using base_helper<ID + 1, Types...>::operator==;
	using base_helper<ID + 1, Types...>::operator<;
	using base_helper<ID + 1, Types...>::typeId;

	virtual int typeId(const T &) const {
		return ID;
	}

	virtual bool operator==(const T &) const {
		return false;
	}

	virtual bool operator<(const T & other) const {
		return this->selfTypeId() < typeId(other);
	}
};

template<typename T, typename... Types>
class base : public base_helper<1, Types...> {
public:
	using base_helper<1, Types...>::operator==;
	using base_helper<1, Types...>::operator<;
	using base_helper<1, Types...>::typeId;

	virtual T* clone() const = 0;

	virtual T* plunder() && = 0;

	bool operator!=(const T& other) const {
		return !(*this == other);
	}

	virtual bool operator==(const T& other) const = 0;

	virtual bool operator<(const T& other) const = 0;
	virtual bool operator>(const T& other) const = 0;

	friend std::ostream& operator<<(std::ostream& os, const T& instance) {
		instance >> os;
		return os;
	}

	virtual void operator>>(std::ostream&) const = 0;

	virtual operator std::string () const = 0;
};

} /* namespace alib */

#endif /* BASE_H_ */
