/*
 * ContainerFromXMLParser.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "ContainerFromXMLParser.hpp"
#include "../sax/ParserException.h"

namespace container {

bool ContainerFromXMLParser::first(std::list<sax::Token>& input) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "set") || isToken(input, sax::Token::TokenType::START_ELEMENT, "vector") || isToken(input, sax::Token::TokenType::START_ELEMENT, "pair") || isToken(input, sax::Token::TokenType::START_ELEMENT, "map")) {
		return true;
	} else {
		return false;
	}
}

Container ContainerFromXMLParser::parseContainer(std::list<sax::Token>& input) const {
	return parseContainer(input, std::set<FEATURES>({FEATURES::SET, FEATURES::VECTOR, FEATURES::PAIR, FEATURES::MAP}));
}

Container ContainerFromXMLParser::parseContainer(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "set")) {
		if(!features.count(FEATURES::SET)) throw exception::AlibException();
		return Container(parseObjectsSet(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "vector")) {
		if(!features.count(FEATURES::VECTOR)) throw exception::AlibException();
		return Container(parseObjectsVector(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "pair")) {
		if(!features.count(FEATURES::PAIR)) throw exception::AlibException();
		return Container(parseObjectsPair(input));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "map")) {
		if(!features.count(FEATURES::MAP)) throw exception::AlibException();
		return Container(parseObjectsMap(input));
	} else {
		throw sax::ParserException(sax::Token("set / vector / pair / map", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

ObjectsSet ContainerFromXMLParser::parseObjectsSet(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "set");

	ObjectsSet set;

	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		set.insert(alib::api<alib::Object>::parse(input));
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, "set");
	return set;
}

ObjectsVector ContainerFromXMLParser::parseObjectsVector(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "vector");

	ObjectsVector vector;

	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		vector.push_back(alib::api<alib::Object>::parse(input));
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, "vector");
	return vector;
}

ObjectsPair ContainerFromXMLParser::parseObjectsPair(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "pair");

	alib::Object first = alib::api<alib::Object>::parse(input);
	alib::Object second = alib::api<alib::Object>::parse(input);

	ObjectsPair pair(first, second);

	popToken(input, sax::Token::TokenType::END_ELEMENT, "pair");
	return pair;
}

ObjectsMap ContainerFromXMLParser::parseObjectsMap(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "map");

	ObjectsMap map;
	while(isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		map.insert(alib::api<std::pair<alib::Object, alib::Object>>::parse(input));
	}

	popToken(input, sax::Token::TokenType::END_ELEMENT, "map");
	return map;
}

} /* namespace container */

