/*
 * GrammarToXMLComposer.cpp
 *
 *  Created on: Nov 11, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarToXMLComposer.h"
#include "../alphabet/BlankSymbol.h"

#include "../Api.hpp"

namespace grammar {

std::list<sax::Token> GrammarToXMLComposer::compose(const GrammarBase& grammar) const {
	std::list<sax::Token> out;
	grammar.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const Grammar& grammar) const {
	std::list<sax::Token> out;
	grammar.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const UnknownGrammar& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("grammar", sax::Token::TokenType::START_ELEMENT));
	
	if (grammar.getNonterminalAlphabet().size() > 0)
		composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());

	if (grammar.getTerminalAlphabet().size() > 0)
		composeTerminalAlphabet(out, grammar.getTerminalAlphabet());

	if (grammar.hasInitialSymbol())
		composeInitialSymbol(out, grammar.getInitialSymbol());

	if (grammar.getRules().size() > 0)
		composeRules(out, grammar);

	out.push_back(sax::Token("grammar", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const LeftLG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LeftLG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token("LeftLG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const LeftRG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LeftRG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token("LeftRG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const RightLG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("RightLG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token("RightLG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const RightRG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("RightRG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token("RightRG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const LG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token("LG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const CFG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("CFG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token("CFG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const EpsilonFreeCFG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("EpsilonFreeCFG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token("EpsilonFreeCFG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const CNF& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("CNF", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token("CNF", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const GNF& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("GNF", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token("GNF", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const CSG& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("CSG", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token("CSG", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const NonContractingGrammar& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("NonContractingGrammar", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);
	composeGeneratesEpsilon(out, grammar.getGeneratesEpsilon());

	out.push_back(sax::Token("NonContractingGrammar", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const ContextPreservingUnrestrictedGrammar& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("ContextPreservingUnrestrictedGrammar", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token("ContextPreservingUnrestrictedGrammar", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> GrammarToXMLComposer::compose(const UnrestrictedGrammar& grammar) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("UnrestrictedGrammar", sax::Token::TokenType::START_ELEMENT));
	
	composeNonterminalAlphabet(out, grammar.getNonterminalAlphabet());
	composeTerminalAlphabet(out, grammar.getTerminalAlphabet());
	composeInitialSymbol(out, grammar.getInitialSymbol());
	composeRules(out, grammar);

	out.push_back(sax::Token("UnrestrictedGrammar", sax::Token::TokenType::END_ELEMENT));
	return out;
}

void GrammarToXMLComposer::composeNonterminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("nonterminalAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("nonterminalAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeTerminalAlphabet(std::list<sax::Token>& out, const std::set<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("terminalAlphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("terminalAlphabet", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeInitialSymbol(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("initialSymbol", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("initialSymbol", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeGeneratesEpsilon(std::list<sax::Token>& out, bool generatesEpsilon) const {
	out.push_back(sax::Token("generatesEpsilon", sax::Token::TokenType::START_ELEMENT));
	if(generatesEpsilon) {
		out.push_back(sax::Token("true", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("true", sax::Token::TokenType::END_ELEMENT));
	} else {
		out.push_back(sax::Token("false", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("false", sax::Token::TokenType::END_ELEMENT));
	}
	out.push_back(sax::Token("generatesEpsilon", sax::Token::TokenType::END_ELEMENT));
}


void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const UnknownGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {
		out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

		if (rule.getLeftHandSide().size() > 0)
			composeRuleLHS(out, rule.getLeftHandSide());

		if (rule.getLeftHandSide().size() > 0)
			composeRuleRHS(out, rule.getRightHandSide());

		out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const UnrestrictedGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const ContextPreservingUnrestrictedGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLContext(out, std::get<0>(rule.first));
			composeRuleSingleSymbolLHS(out, std::get<1>(rule.first));
			composeRuleRContext(out, std::get<2>(rule.first));
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const NonContractingGrammar& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const CSG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleLContext(out, std::get<0>(rule.first));
			composeRuleSingleSymbolLHS(out, std::get<1>(rule.first));
			composeRuleRContext(out, std::get<2>(rule.first));
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const GNF& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleGNFRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const CNF& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleOneOrTwoSymbolsRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const EpsilonFreeCFG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const CFG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const LG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleLGRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const RightLG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleRightLGRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const RightRG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleOneOrTwoSymbolsRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const LeftLG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleLeftLGRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRules(std::list<sax::Token>& out, const LeftRG& grammar) const {
	out.push_back(sax::Token("rules", sax::Token::TokenType::START_ELEMENT));
	for (const auto& rule : grammar.getRules()) {

		for(const auto& rhs : rule.second) {
			out.push_back(sax::Token("rule", sax::Token::TokenType::START_ELEMENT));

			composeRuleSingleSymbolLHS(out, rule.first);
			composeRuleOneOrTwoSymbolsRHS(out, rhs);

			out.push_back(sax::Token("rule", sax::Token::TokenType::END_ELEMENT));
		}
	}

	out.push_back(sax::Token("rules", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLContext(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("lContext", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("lContext", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("lhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("lhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleRContext(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("rContext", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("rContext", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleSingleSymbolLHS(std::list<sax::Token>& out, const alphabet::Symbol& symbol) const {
	out.push_back(sax::Token("lhs", sax::Token::TokenType::START_ELEMENT));
	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	out.push_back(sax::Token("lhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleRHS(std::list<sax::Token>& out, const std::vector<alphabet::Symbol>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.size() == 0) {
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
		out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
	} else for (const auto& symbol : symbols) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleOneOrTwoSymbolsRHS(std::list<sax::Token>& out, const std::variant<alphabet::Symbol, std::pair<alphabet::Symbol, alphabet::Symbol>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<alphabet::Symbol>()) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbols.get<alphabet::Symbol>()));
	} else {
		const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = symbols.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(rhs.first));
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(rhs.second));
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleGNFRHS(std::list<sax::Token>& out, const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));

	out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbols.first));

	for (const auto& symbol : symbols.second) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}

	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<std::vector<alphabet::Symbol>>()) {
		const std::vector<alphabet::Symbol>& rhs = symbols.get<std::vector<alphabet::Symbol>>();
		if(rhs.size() == 0) {
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
		} else for (const auto& symbol : symbols.get<std::vector<alphabet::Symbol>>()) {
			out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
		}
	} else {
		const std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>& rhs = symbols.get<std::tuple<std::vector<alphabet::Symbol>, alphabet::Symbol, std::vector<alphabet::Symbol>>>();
		for (const auto& symbol : std::get<0>(rhs)) {
			out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
		}
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(std::get<1>(rhs)));
		for (const auto& symbol : std::get<2>(rhs)) {
			out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
		}
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleLeftLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<std::vector<alphabet::Symbol>>()) {
		const std::vector<alphabet::Symbol>& rhs = symbols.get<std::vector<alphabet::Symbol>>();
		if(rhs.size() == 0) {
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
		} else for (const auto& symbol : symbols.get<std::vector<alphabet::Symbol>>()) {
			out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
		}
	} else {
		const std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>& rhs = symbols.get<std::pair<alphabet::Symbol, std::vector<alphabet::Symbol>>>();
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(rhs.first));
		for (const auto& symbol : rhs.second) {
			out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
		}
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

void GrammarToXMLComposer::composeRuleRightLGRHS(std::list<sax::Token>& out, const std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>>& symbols) const {
	out.push_back(sax::Token("rhs", sax::Token::TokenType::START_ELEMENT));
	if(symbols.is<std::vector<alphabet::Symbol>>()) {
		const std::vector<alphabet::Symbol>& rhs = symbols.get<std::vector<alphabet::Symbol>>();
		if(rhs.size() == 0) {
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
			out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
		} else {
			for (const auto& symbol : symbols.get<std::vector<alphabet::Symbol>>()) {
				out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
			}
		}
	} else {
		const std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>& rhs = symbols.get<std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>>();
		for (const auto& symbol : rhs.first) {
			out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
		}
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(rhs.second));
	}
	out.push_back(sax::Token("rhs", sax::Token::TokenType::END_ELEMENT));
}

} /* namespace grammar */

