/*
 * LabelFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelFromStringLexer.h"

namespace label {

LabelFromStringLexer::LabelFromStringLexer(std::istream& in) : m_In(in) {
	m_Current.type = TokenType::ERROR;
	m_Current.value = "";
}

LabelFromStringLexer& LabelFromStringLexer::next() {
	if(m_Current.type == TokenType::TEOF) return *this;
	char character;
	m_Current.value = "";

L0:
	character = m_In.get();
	if(m_In.eof()) {
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		goto L0;
	} else if(character == '[') {
		m_Current.type = TokenType::SET_BEGIN;
		m_Current.value += character;
		return *this;
	} else if(character == ']') {
		m_Current.type = TokenType::SET_END;
		m_Current.value += character;
		return *this;
	} else if(character == '<') {
		m_Current.type = TokenType::PAIR_BEGIN;
		m_Current.value += character;
		return *this;
	} else if(character == '>') {
		m_Current.type = TokenType::PAIR_END;
		m_Current.value += character;
		return *this;
	} else if(character == ',') {
		m_Current.type = TokenType::COMMA;
		m_Current.value += character;
		return *this;
	} else {
		m_In.putback(character);
		while(!m_Current.value.empty()) {
			m_In.putback(m_Current.value.back());
			m_Current.value.pop_back();
		}
		m_In.clear();
		m_Current.type = TokenType::ERROR;
		return *this;
	}
}

LabelFromStringLexer::Token LabelFromStringLexer::token() {
	return m_Current;
}

} /* namespace label */
