/*
 * LabelPairLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_PAIR_LABEL_H_
#define LABEL_PAIR_LABEL_H_

#include <string>
#include <ostream>

#include "Label.h"
#include "LabelBase.h"
#include <set>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LabelPairLabel : public std::acceptor<LabelPairLabel, VisitableLabelBase, std::acceptor<LabelPairLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	std::pair<Label, Label> label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabelPairLabel(const std::pair<Label, Label>& label);

	explicit LabelPairLabel(std::pair<Label, Label>&& label);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	virtual bool operator<(const alib::ObjectBase& other) const;

	virtual bool operator>(const alib::ObjectBase& other) const;

	virtual bool operator==(const alib::ObjectBase& other) const;

	/**
	 * @return name of the symbol
	 */
	const std::pair<Label, Label>& getData() const;

	virtual bool operator <(const LabelPairLabel& other) const;
	virtual bool operator ==(const LabelPairLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual operator std::string () const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace label */

#endif /* LABEL_PAIR_LABEL_H_ */

