/*
 * Integer.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_INTEGER_H_
#define PRIMITIVE_INTEGER_H_

#include <ostream>

#include "PrimitiveBase.h"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Integer : public std::acceptor<Integer, VisitablePrimitiveBase, std::acceptor<Integer, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	int data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Integer(int data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	virtual bool operator<(const alib::ObjectBase& other) const;
	
	virtual bool operator>(const alib::ObjectBase& other) const;

	virtual bool operator==(const alib::ObjectBase& other) const;

	/**
	 * @return name of the symbol
	 */
	int getData() const;

	virtual bool operator <(const Integer& other) const;
	
	virtual bool operator ==(const Integer& other) const;
	
	virtual void operator>>(std::ostream&) const;

	virtual operator std::string() const;

	virtual int selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace primitive */

#endif /* PRIMITIVE_INTEGER_H_ */
