/*
 * RegExpFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_FROM_XML_PARSER_H_
#define REG_EXP_FROM_XML_PARSER_H_

#include "../sax/FromXMLParserHelper.h"
#include "RegExp.h"
#include "RegExpFeatures.h"
#include "unbounded/UnboundedRegExpElements.h"
#include "formal/FormalRegExpElements.h"
#include "../sax/Token.h"
#include "../alphabet/Symbol.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace regexp {

/**
 * Parser used to get RegExp from XML parsed into list of tokens.
 */
class RegExpFromXMLParser : public sax::FromXMLParserHelper {
	void parseAlphabet(std::list<sax::Token>& input, UnboundedRegExp& regexp) const;
	void parseAlphabet(std::list<sax::Token>& input, FormalRegExp& regexp) const;

	UnboundedRegExpElement* parseUnboundedRegExpElement(std::list<sax::Token>& input) const;

	UnboundedRegExpEpsilon* parseUnboundedRegExpEpsilon(std::list<sax::Token>& input) const;
	UnboundedRegExpEmpty* parseUnboundedRegExpEmpty(std::list<sax::Token>& input) const;
	UnboundedRegExpIteration* parseUnboundedRegExpIteration(std::list<sax::Token> &input) const;
	UnboundedRegExpAlternation* parseUnboundedRegExpAlternation(std::list<sax::Token> &input) const;
	UnboundedRegExpConcatenation* parseUnboundedRegExpConcatenation(std::list<sax::Token> &input) const;

	FormalRegExpElement* parseFormalRegExpElement(std::list<sax::Token>& input) const;

	FormalRegExpEpsilon* parseFormalRegExpEpsilon(std::list<sax::Token>& input) const;
	FormalRegExpEmpty* parseFormalRegExpEmpty(std::list<sax::Token>& input) const;
	FormalRegExpIteration* parseFormalRegExpIteration(std::list<sax::Token> &input) const;
	FormalRegExpAlternation* parseFormalRegExpAlternation(std::list<sax::Token> &input) const;
	FormalRegExpConcatenation* parseFormalRegExpConcatenation(std::list<sax::Token> &input) const;

	RegExp parseRegExp(std::list<sax::Token>& input) const;
	RegExp parseRegExp(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;

	UnboundedRegExp parseUnboundedRegExp(std::list<sax::Token>& input) const;
	FormalRegExp parseFormalRegExp(std::list<sax::Token>& input) const;

	template<typename T> friend class alib::api;
public:
	bool first(std::list<sax::Token>& input) const;
};

} /* namespace regexp */

#endif /* REG_EXP_FROM_XML_PARSER_H_ */
