/*
 * RegExpToXMLComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_TO_XML_COMPOSER_H_
#define REG_EXP_TO_XML_COMPOSER_H_

#include "RegExp.h"
#include "unbounded/UnboundedRegExpElements.h"
#include "formal/FormalRegExpElements.h"
#include "../std/visitor.hpp"
#include "../sax/Token.h"

namespace alib {

template<typename T>
struct api;

} /* namespace alib */

namespace regexp {

/**
 * This class contains methods to print XML representation of regular expression to the output stream.
 */
class RegExpToXMLComposer :public UnboundedRegExpElement::const_visitor_type, public FormalRegExpElement::const_visitor_type {
	void Visit(void*, const UnboundedRegExpAlternation& alternation) const;
	void Visit(void*, const UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void*, const UnboundedRegExpIteration& iteration) const;
	void Visit(void*, const UnboundedRegExpSymbol& symbol) const;
	void Visit(void*, const UnboundedRegExpEpsilon& epsilon) const;
	void Visit(void*, const UnboundedRegExpEmpty& empty) const;
	
	
	void Visit(void*, const FormalRegExpAlternation& alternation) const;
	void Visit(void*, const FormalRegExpConcatenation& concatenation) const;
	void Visit(void*, const FormalRegExpIteration& iteration) const;
	void Visit(void*, const FormalRegExpSymbol& symbol) const;
	void Visit(void*, const FormalRegExpEpsilon& epsilon) const;
	void Visit(void*, const FormalRegExpEmpty& empty) const;


	/**
	 * Composes XML token representation of RegExp.
	 * @param regexp RegExp to print
	 * @returns list of xml tokens.
	 */
	std::list<sax::Token> compose(const RegExpBase& regexp) const;

	/**
	 * Composes XML token representation of RegExp.
	 * @param regexp RegExp to print
	 * @returns list of xml tokens.
	 */
	std::list<sax::Token> compose(const RegExp& regexp) const;

	std::list<sax::Token> compose(const UnboundedRegExp& regexp) const;
	std::list<sax::Token> compose(const FormalRegExp& regexp) const;

	template<typename T> friend class alib::api;
};

} /* namespace regexp */

#endif /* REG_EXP_TO_XML_COMPOSER_H_ */
