/*
 * UnboundedRegExpConcatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#ifndef UNBOUNDED_REG_EXP_CONCATENATION_H_
#define UNBOUNDED_REG_EXP_CONCATENATION_H_

#include <vector>
#include "UnboundedRegExpElement.h"

namespace regexp {

class RegExpOptimize;

/**
 * Represents concatenation operator in the regular expression. Contains list of UnboundedRegExpElement
 * as operands of the operator.
 */
class UnboundedRegExpConcatenation: public std::acceptor<UnboundedRegExpConcatenation, UnboundedRegExpElement, UnboundedRegExpElement> {
protected:
	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	virtual FormalRegExpElement* cloneAsFormal() const;


	std::vector<UnboundedRegExpElement*> elements;
	
	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc UnboundedRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const UnboundedRegExp * regexp );
	
	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	explicit UnboundedRegExpConcatenation();
	
	UnboundedRegExpConcatenation(const UnboundedRegExpConcatenation& other);
	UnboundedRegExpConcatenation(UnboundedRegExpConcatenation&& other) noexcept;
	UnboundedRegExpConcatenation& operator =(const UnboundedRegExpConcatenation& other);
	UnboundedRegExpConcatenation& operator =(UnboundedRegExpConcatenation&& other);
	virtual ~UnboundedRegExpConcatenation() noexcept;

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* clone() const;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	virtual UnboundedRegExpElement* plunder() &&;

	
	/**
	 * @return elements
	 */
	const std::vector<const UnboundedRegExpElement*> & getElements() const;

	/**
	 * @return elements
	 */
	const std::vector<UnboundedRegExpElement*> & getElements();

	/**
	 * @param element to append
	 */
	void appendElement(const UnboundedRegExpElement& element);
	
	void appendElement(UnboundedRegExpElement&& element);

	virtual bool operator<(const UnboundedRegExpElement&) const;
	virtual bool operator==(const UnboundedRegExpElement&) const;
	virtual bool operator>(const UnboundedRegExpElement&) const;

	virtual bool operator<(const UnboundedRegExpConcatenation&) const;
	virtual bool operator==(const UnboundedRegExpConcatenation&) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;
	
	friend class RegExpOptimize;
};

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_CONCATENATION_H_ */
