/*
 * StringToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringToXMLComposer.h"
#include "CyclicString.h"
#include "LinearString.h"
#include "Epsilon.h"

#include "../Api.hpp"

namespace string {

std::list<sax::Token> StringToXMLComposer::compose(const String& string) const {
	std::list<sax::Token> out;
	string.getData().Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> StringToXMLComposer::compose(const StringBase& string) const {
	std::list<sax::Token> out;
	string.Accept((void*) &out, alib::ToXMLComposers::toXMLComposers);
	return out;
}

std::list<sax::Token> StringToXMLComposer::compose(const LinearString& string) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("LinearString", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol: string.getAlphabet()) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));

	for(const auto& symbol : string.getContent()) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("LinearString", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> StringToXMLComposer::compose(const CyclicString& string) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("CyclicString", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol: string.getAlphabet()) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));

	for(const auto& symbol : string.getContent()) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("CyclicString", sax::Token::TokenType::END_ELEMENT));
	return out;
}

std::list<sax::Token> StringToXMLComposer::compose(const Epsilon& string) const {
	std::list<sax::Token> out;
	out.push_back(sax::Token("Epsilon", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
	for (const auto& symbol: string.getAlphabet()) {
		out.splice(out.end(), alib::api<alphabet::Symbol>::compose(symbol));
	}
	out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));
	out.push_back(sax::Token("Epsilon", sax::Token::TokenType::END_ELEMENT));
	return out;
}

} /* namespace string */
